@@ -4,15 +4,28 @@ import os
 import io
 import sys
 import pkgutil
+from pkgutil import iter_modules
 
 import pkg_resources  # part of setuptools
 version = pkg_resources.require("bridge-utils")[0].version
 
-from .pbn_parser import importPBN, ParseError
+# https://stackoverflow.com/questions/16981921/relative-imports-in-python-3/28154841
+import repackage
+repackage.up()
+
+from bridge_utils.pbn2html.pbn_parser import importPBN, ParseError
 #from pbn_parser import importPBN, ParseError
 
 from string import Template
 
+try:
+    from ddstable import ddstable
+except ImportError as error:
+    pass
+
+def module_exists(module_name):
+    return module_name in (name for loader, name, ispkg in iter_modules())
+
 language="Chinese"
 
 html_template="""
@@ -94,6 +107,35 @@ auction_template="""
           $auction
 """
 
+pbn_template="""
+<table style="border: 1px solid;font-size:small;" cellspacing="0px" cellpadding="0px" align="right" bgcolor="#F8F8F8">
+    <tbody>
+      <tr bgcolor="#FFFFFF">
+        <td style="border-style: none;"></td>
+        <td style="border-style: none;" align="center"><small>&nbsp;<font color="black">&clubs;</font>&nbsp;</small></td>
+        <td style="border-style: none;" align="center"><small>&nbsp;<font color="red">&diams;</font>&nbsp;</small></td>
+        <td style="border-style: none;" align="center"><small>&nbsp;<font color="red">&hearts;</font>&nbsp;</small></td>
+        <td style="border-style: none;" align="center"><small>&nbsp;<font color="black">&spades;</font>&nbsp;</small></td>
+        <td style="border-style: none;" align="center"><small><font color="black">NT</font></small></td>
+      </tr>
+      $tr
+    </tbody>
+  </table>
+"""
+
+def read_template(template_file):
+    # read local first (debug), then module
+    try:
+        template = open(template_file, "r",encoding="utf-8").read()
+        src = Template(template)
+    except FileNotFoundError:
+        if __name__ == "__main__":
+            raise IOError("can't find file, debug?")
+        else:
+            template = pkgutil.get_data(__name__,template_file)
+            src = Template(template.decode('utf-8'))
+    return src
+
 def html_card(cards):
     src = Template(card_template)
     all = {
@@ -220,6 +262,24 @@ def html_auction(auction, section_auction):
     all = { "auction": css}
     return src.safe_substitute(all)
 
+
+def html_pbn(pbn):
+    tables = ddstable.get_ddstable(pbn)
+    src = Template(pbn_template)
+    tr = ""
+    card_suit=["C", "D", "H", "S", "NT"]
+    for each in tables.keys():
+        td = ""
+        for suit in card_suit:
+            trick=tables[each][suit]
+            trick_str = "-"
+            if trick > 6:
+                trick_str = str(trick-6)
+            td += "<td style='border-style: none;' align='center'>%s</td>\n" % trick_str
+        tr += "<tr><td style='border-style: none;' align='center'><font color='#008080'>%s</font></td>\n%s</tr>" % ( each, td)
+    all = { "tr": tr}
+    return src.safe_substitute(all)
+    
 def pbn2html(pbn_file):
     all = {
         "generated" : "2021-05-14"
@@ -250,26 +310,7 @@ def pbn2html(pbn_file):
             all["board"] = html_board(tags["Vulnerable"],tags["Dealer"])
             all["extra"] = html_extra(tags["Contract"],tags["Declarer"],tags["Lead"])
             all["auction"] = html_auction(tags["Auction"], section_auction)
-            # hacked solution to check whether it is module or local
-            # windows module
-            # _: C:/Users/rdccaiy/AppData/Local/Programs/Python/Python38/Scripts/pbn2html
-            # unix module
-            # _: /home/rdccaiy/py3env/bin/pbn2html
-            # local (python3.8)
-            # _: ./pbn2html.py
-            #for item, value in os.environ.items():
-            #    print('{}: {}'.format(item, value))
-            if sys.platform.startswith('win'):
-                template = pkgutil.get_data(__name__,'template.html')
-                src = Template(template.decode('utf-8'))
-            else:
-                if "pbn2html.py" in os.environ.get("_"):
-                    template = open("template.html", "r",encoding="utf-8").read()
-                    src = Template(template)
-                else:
-                    template = pkgutil.get_data(__name__,'template.html')
-                    src = Template(template.decode('utf-8'))
-
+            src = read_template("template.html")
             result += src.safe_substitute(all)
         except ParseError as error:
             print(error)
@@ -310,7 +351,7 @@ def pbn_html_auction(pbn):
     result = src.safe_substitute(all)
     return result
 
-def pbn_html_deal(pbn, cards="NESW", ll="", ul="", ur=""):
+def pbn_html_deal(pbn, cards="NESW", ll="", ul="", ur="", dds=False):
     all = {}
     tags = pbn["tags"]
     hands = pbn["hands"]
@@ -332,6 +373,13 @@ def pbn_html_deal(pbn, cards="NESW", ll="", ul="", ur=""):
             if card == "S":
                 all["south"] = empty_hand
 
+    if dds and module_exists("ddstable"):
+        PBN = bytes(pbn["tags"]["Deal"],encoding="raw_unicode_escape")
+        # print(PBN)
+        all["lr"] = html_pbn(PBN)
+    else:
+        all["lr"] = ""
+
     all["board"] = html_board(tags["Vulnerable"],tags["Dealer"])
     if ul == "":
         all["ul"] = html_extra(tags["Contract"],tags["Declarer"],tags["Lead"])
@@ -340,8 +388,7 @@ def pbn_html_deal(pbn, cards="NESW", ll="", ul="", ur=""):
     all["ll"] = html_info(ll, bottom=True)
     all["ur"] = html_info(ur)
 
-    template = pkgutil.get_data(__name__,'deal_template.html')
-    src = Template(template.decode('utf-8'))
+    src = read_template('deal_template.html')
     #template = open("deal_template.html", "r", encoding="utf-8").read()
     #src = Template(template)
     result = src.safe_substitute(all)
@@ -374,7 +421,7 @@ def main():
             #print(pbn_html_all(pbn))
             #print(pbn_html_auction(pbn))
             
-            #print(pbn_html_deal(pbn, cards="NS"))
+            pbn_html_deal(pbn, cards="NS", dds=True)
             #print(pbn_html_deal(pbn, cards="EW"))
             #print(pbn_html_deal(pbn, cards="NEWS"))
             #print(pbn_html_deal(pbn, ul=" ", ur="群组赛1209&牌号 4/8", ll="NS 4/12&EW 0"))
