@@ -68,14 +68,14 @@ def fixcontract(contract):
 
 def xin2pbn(src_url, output):
     #print("wahtttttttt")
-    #print(url)
-    url = src_url.replace("&amp;","&").replace(";","%3B")
+    print(src_url)
+    url = src_url.replace(";N:#1111","").replace("&amp;","&").replace(";","%3B")
     o = urlparse(url)
     all = urllib.parse.parse_qs(o.query)
-    #print(all)
+    print(all)
     for k in all:
         all[k] = all[k][0]
-    #print(all)
+    print(all)
     all["leader"] = DECLARE2LEADER[all["declarer"]]
     all["auction"] = bidlog2auction(all["bidlog"])
     if "playlog" in all:
